require 'spec_helper'

describe Citizen do

  before { @citizen = Citizen.new(name: "Example citizen", login: "user@example.com",
                                  password: "password", password_confirmation: "password") }

  subject { @citizen }

  it { should respond_to(:name) }
  it { should respond_to(:login) }
  it { should respond_to(:password_digest) }
  it { should respond_to(:password) }
  it { should respond_to(:password_confirmation) }
  it { should respond_to(:remember_token) }
  it { should respond_to(:authenticate) }
  it { should respond_to(:admin) }
  it { should respond_to(:posts) }
  it { should respond_to(:comments) }

  it { should be_valid }
  it { should_not be_admin }

  describe "with admin attribute set to 'true'" do
    before do
      @citizen.save!
      @citizen.toggle!(:admin)
    end

    it { should be_admin }
  end

  describe "when name is not present" do
    before { @citizen.name = " " }
    it { should_not be_valid }
  end

  describe "when name is to long" do
    before { @citizen.name = "a" * 51 }
    it { should_not be_valid }
  end

  describe "when login is not present" do
    before { @citizen.login = " " }
    it { should_not be_valid }
  end

  describe "when login format is invalid" do
    it "should be invalid" do
      logins = %w[user@foo,com user_at_foo.org example.user@foo.
                     foo@bar_baz.com foo@bar+baz.com foo@bar..com]

      logins.each do |invalid_login|
        @citizen.login = invalid_login
        expect(@citizen).not_to be_valid
      end
    end
  end

  describe "when login format is valid" do
    it "should be valid" do
      logins = %w[user@foo.COM A_US-ER@f.b.org frst.lst@foo.jp a+b@baz.cn]

      logins.each do |valid_login|
        @citizen.login = valid_login
        expect(@citizen).to be_valid
      end
    end
  end

  describe "when login have mixed case" do
    let(:mixed_case_login) { "Foo@ExAMPle.CoM" }

    it "should be saved as all lower-case" do
      @citizen.login = mixed_case_login
      @citizen.save
      expect(@citizen.reload.login).to eq mixed_case_login.downcase
    end
  end

  describe "when login is already taken" do
    before do
      citizen_with_same_login = @citizen.dup
      citizen_with_same_login.login = @citizen.login.upcase
      citizen_with_same_login.save
    end

    it { should_not be_valid }
  end

  describe "when password is not present" do
    before do
      @citizen = Citizen.new(name: "Example", login: "example@example.com",
                             password: " ", password_confirmation: " ")
    end

    it { should_not be_valid }
  end

  describe "when password doesn't match confirmation" do
    before { @citizen.password_confirmation = "mismatch" }
    it { should_not be_valid }
  end

  describe "with a password that is too short" do
    before { @citizen.password = @citizen.password_confirmation = "a" * 5 }
    it { should be_invalid }
  end

  describe "return value of authenticate method" do
    before { @citizen.save } # we need it in the DB for find_by to work
    let(:found_citizen) { Citizen.find_by(login: @citizen.login) }

    describe "with valid password" do
      it { should eq found_citizen.authenticate(@citizen.password) }
    end

    describe "with invalid password" do
      let(:citizen_for_invalid_password) { found_citizen.authenticate("wrong_password") }

      it { should_not eq citizen_for_invalid_password }
      # authenticate returns the corresponding object or false
      specify { expect(citizen_for_invalid_password).to be_false }
    end
  end

  describe "remember token" do
    before { @citizen.save } # we need to create the citizen saving it
    its (:remember_token) { should_not be_blank }
  end

  describe "post associations" do

    before { @citizen.save }
    let!(:older_post) { FactoryGirl.create(:post, citizen: @citizen, created_at: 1.day.ago) }
    let!(:newer_post) { FactoryGirl.create(:post, citizen: @citizen, created_at: 1.hour.ago) }

    it "should have the right posts in the right order (reverse date)" do
      expect(@citizen.posts.to_a).to eq [newer_post, older_post]
    end
  end
end
